/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: addr_editor.c
 *            DESCRIPTION: edit the Palm Pilot address list
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: 
 *      PRIVATE FUNCTIONS: 
 *                       : 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_ADDR_EDITOR_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPCloseAddrEditor();


/* === Private routine prototypes === */
static int FindPhoneLabel();
static void ProcessChanges();
static void CategoryChange();
static void PhoneLabelChange1();
static void PhoneLabelChange2();
static void PhoneLabelChange3();
static void PhoneLabelChange4();
static void PhoneLabelChange5();

/* === Global Variables === */
extern AppRes		app_res;
extern StickyNote	*notes;
extern GtkWidget	*toplevel;

extern PilotAddr_T			*Pilot_Address;
extern PilotCategory_T		*Pilot_Addr_Categories;


/* === Static Variables === */
static GtkWidget	*addr_editor_dialog = NULL;
static GtkWidget	*lastname_f, *firstname_f, *title_f, *company_f;
static GtkWidget	*address_f, *city_f, *state_f, *zipcode_f, *country_f;
static GtkWidget	*phone1, *phone2, *phone3, *phone4, *phone5;
static GtkWidget	*phone1_f, *phone2_f, *phone3_f, *phone4_f, *phone5_f;
static GtkWidget	*custom1_f, *custom2_f, *custom3_f, *custom4_f;
static GtkWidget	*text_f;

static PilotCategory_T	*category_current;

static int	phoneLabel[5] = { 0, 1, 2, 3, 4 };

static char	*phoneLabels[] = {
	"Work",
	"Home",
	"Fax",
	"Other",
	"E-mail",
	"Main",
	"Pager",
	"Mobile",
	NULL
};

/*========================================================================
 *	Name:			XPAddressEditor()
 *	Prototype:	void XPAddressEditor()
 *					
 *	Description:
 * Setup and present the Address Editor window.
 *
 * Input Arguments:
 *	PilotAddr_T		*ptr		If NULL, we're in Add mode, otherwise edit the
 *									entry pointed to by ptr
 *
 *	Notes:
 *========================================================================*/
void
XPAddressEditor(
	PilotAddr_T		*ptr
)
{
	GtkWidget		*vbox, *hbox, *table;
	GtkWidget		*label;
	GtkWidget		*accept_button; 
	GtkWidget		*close_button; 
	GtkWidget		*notebook;
	GtkWidget		*alignment;
	GtkWidget		*submenu, *menuitem;
	GtkWidget		*lastname, *firstname, *title, *company;
	GtkWidget		*address, *city, *state, *zipcode, *country;
	GtkWidget		*custom1, *custom2, *custom3, *custom4;
	GtkWidget		*category_f;
	GtkWidget		*frame;
	GtkWidget		*tabLabel_1;
	GtkWidget		*tabLabel_2;
	GtkWidget		*tabLabel_3;
	GShell_T			gshell;

	int				index, category_index;

	PilotCategory_T	*category_link;


	/*
	 * Allow only one window at a time.
	 */
	if ( addr_editor_dialog != NULL )
		return;


	/*
	 * The addr editor dialog parent window.
	 */
	addr_editor_dialog = 
		XPCreateGenericShell(AddrEditorDialog, 
			ACCEPT_BUTTON|CLOSE_BUTTON,
			&gshell, FALSE );

	/* Save the button widget ids */
	vbox = gshell.vbox;
	close_button = gshell.cancel;
	accept_button = gshell.accept;

	gtk_signal_connect (GTK_OBJECT (addr_editor_dialog), "destroy",
			GTK_SIGNAL_FUNC (XPCloseAddrEditor), NULL);

	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (XPCloseAddrEditor), NULL);

	/*
	 * If no link was passed to us, we need to change the Accept button to
	 * say "Add".
	 */
	if ( ptr == NULL )
	{
		gtk_label_set_text(GTK_LABEL(GTK_BIN(accept_button)->child),
			XP_S_ADD_ENTRY);
		gtk_signal_connect (GTK_OBJECT (accept_button), "clicked",
			GTK_SIGNAL_FUNC (ProcessChanges), NULL);
	}
	else
	{
		gtk_signal_connect (GTK_OBJECT (accept_button), "clicked",
			GTK_SIGNAL_FUNC (ProcessChanges), ptr);
	}

	/* Add a category menu */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 4);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_CATEGORY_NAME);
	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 4);
	gtk_widget_show(label);

	submenu = gtk_menu_new();
	category_link = Pilot_Addr_Categories;
	category_current = NULL;
	category_index = 0;
	index = 0;
	while ( category_link != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(category_link->name);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(CategoryChange), (gpointer) category_link);
		if (ptr)
		{
			if (strcmp(category_link->name, ptr->category) == 0)
			{
				category_index = index;
				category_current = category_link;
			}
		}
		index++;
		category_link = category_link->next;
	}
	category_f = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (category_f), submenu);
	gtk_box_pack_start (GTK_BOX (hbox), category_f, TRUE, TRUE, 4);
	gtk_widget_show(category_f);

	if (ptr)
	{
		gtk_option_menu_set_history (GTK_OPTION_MENU (category_f), 
			category_index);
	}
	else
	{
		index = 0;
		category_link = Pilot_Addr_Categories;
		while ( category_link != NULL )
		{
			if (strcasecmp(category_link->name, XP_S_CAT_UNFILED) == 0)
			{
				gtk_option_menu_set_history (GTK_OPTION_MENU (category_f), index);
				category_current = category_link;
			}
			index++;
			category_link = category_link->next;
		}
	}

	/*
	 * Create a 3 tab notebook.
	 */
	notebook = gtk_notebook_new();
	gtk_box_pack_start (GTK_BOX (vbox), notebook, TRUE, TRUE, 4);

	/*
	 * Populate the first tab with the name, title, company, address info
	 * and a button for editing the Note.
	 */
	tabLabel_1 = gtk_label_new(XP_S_ADDR_EDIT_PG_1);
	table = gtk_table_new(9,2,FALSE);

	alignment = gtk_alignment_new(0,0,0,0);
	lastname = gtk_label_new(XP_S_LAST_NAME);
	lastname_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 0,1, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), lastname);
	gtk_table_attach_defaults(GTK_TABLE(table), lastname_f, 1,2, 0,1);
	gtk_widget_show(alignment);
	gtk_widget_show(lastname);
	gtk_widget_show(lastname_f);
	gtk_label_set_justify(GTK_LABEL(lastname), GTK_JUSTIFY_LEFT);

	alignment = gtk_alignment_new(0,0,0,0);
	firstname = gtk_label_new(XP_S_FIRST_NAME);
	firstname_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 1,2, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), firstname);
	gtk_table_attach_defaults(GTK_TABLE(table), firstname_f, 1,2, 1,2);
	gtk_widget_show(alignment);
	gtk_widget_show(firstname);
	gtk_widget_show(firstname_f);

	alignment = gtk_alignment_new(0,0,0,0);
	title = gtk_label_new(XP_S_TITLE);
	title_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 2,3, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), title);
	gtk_table_attach_defaults(GTK_TABLE(table), title_f, 1,2, 2,3);
	gtk_widget_show(alignment);
	gtk_widget_show(title);
	gtk_widget_show(title_f);

	alignment = gtk_alignment_new(0,0,0,0);
	company = gtk_label_new(XP_S_COMPANY);
	company_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 3,4, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), company);
	gtk_table_attach_defaults(GTK_TABLE(table), company_f, 1,2, 3,4);
	gtk_widget_show(alignment);
	gtk_widget_show(company);
	gtk_widget_show(company_f);

	alignment = gtk_alignment_new(0,0,0,0);
	address = gtk_label_new(XP_S_ADDRESS);
	address_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 4,5, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), address);
	gtk_table_attach_defaults(GTK_TABLE(table), address_f, 1,2, 4,5);
	gtk_widget_show(alignment);
	gtk_widget_show(address);
	gtk_widget_show(address_f);

	alignment = gtk_alignment_new(0,0,0,0);
	city = gtk_label_new(XP_S_CITY);
	city_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 5,6, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), city);
	gtk_table_attach_defaults(GTK_TABLE(table), city_f, 1,2, 5,6);
	gtk_widget_show(alignment);
	gtk_widget_show(city);
	gtk_widget_show(city_f);

	alignment = gtk_alignment_new(0,0,0,0);
	state = gtk_label_new(XP_S_STATE);
	state_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 6,7, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), state);
	gtk_table_attach_defaults(GTK_TABLE(table), state_f, 1,2, 6,7);
	gtk_widget_show(alignment);
	gtk_widget_show(state);
	gtk_widget_show(state_f);

	alignment = gtk_alignment_new(0,0,0,0);
	zipcode = gtk_label_new(XP_S_ZIPCODE);
	zipcode_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 7,8, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), zipcode);
	gtk_table_attach_defaults(GTK_TABLE(table), zipcode_f, 1,2, 7,8);
	gtk_widget_show(alignment);
	gtk_widget_show(zipcode);
	gtk_widget_show(zipcode_f);

	alignment = gtk_alignment_new(0,0,0,0);
	country = gtk_label_new(XP_S_COUNTRY);
	country_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 8,9, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), country);
	gtk_table_attach_defaults(GTK_TABLE(table), country_f, 1,2, 8,9);
	gtk_widget_show(alignment);
	gtk_widget_show(country);
	gtk_widget_show(country_f);

	gtk_widget_show(table);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, tabLabel_1);

	if (ptr)
	{
		gtk_entry_set_text(GTK_ENTRY(lastname_f), ptr->last);
		gtk_entry_set_text(GTK_ENTRY(firstname_f), ptr->first);
		gtk_entry_set_text(GTK_ENTRY(title_f), ptr->title);
		gtk_entry_set_text(GTK_ENTRY(company_f), ptr->company);
		gtk_entry_set_text(GTK_ENTRY(address_f), ptr->address);
		gtk_entry_set_text(GTK_ENTRY(city_f), ptr->city);
		gtk_entry_set_text(GTK_ENTRY(state_f), ptr->state);
		gtk_entry_set_text(GTK_ENTRY(zipcode_f), ptr->zip);
		gtk_entry_set_text(GTK_ENTRY(country_f), ptr->country);
	}

	/*
	 * Populate the second tab with the phone/email/etc info.
	 */
	tabLabel_2 = gtk_label_new(XP_S_ADDR_EDIT_PG_2);
	table = gtk_table_new(5,2,FALSE);

	submenu = gtk_menu_new();
	index=0;
	while ( phoneLabels[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(phoneLabels[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PhoneLabelChange1), (gpointer) index);
		index++;
	}
	phone1 = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (phone1), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (phone1), 0);

	phone1_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), phone1, 0,1, 0,1, GTK_EXPAND, GTK_EXPAND,4,2);
	gtk_table_attach_defaults(GTK_TABLE(table), phone1_f, 1,2, 0,1);
	gtk_widget_show(alignment);
	gtk_widget_show(phone1);
	gtk_widget_show(phone1_f);

	alignment = gtk_alignment_new(0,0,0,0);
	submenu = gtk_menu_new();
	index=0;
	while ( phoneLabels[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(phoneLabels[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PhoneLabelChange2), (gpointer) index);
		index++;
	}
	phone2 = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (phone2), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (phone2), 1);

	phone2_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), phone2, 0,1, 1,2, GTK_EXPAND, GTK_EXPAND,4,2);
	gtk_table_attach_defaults(GTK_TABLE(table), phone2_f, 1,2, 1,2);
	gtk_widget_show(alignment);
	gtk_widget_show(phone2);
	gtk_widget_show(phone2_f);

	alignment = gtk_alignment_new(0,0,0,0);
	submenu = gtk_menu_new();
	index=0;
	while ( phoneLabels[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(phoneLabels[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PhoneLabelChange3), (gpointer) index);
		index++;
	}
	phone3 = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (phone3), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (phone3), 2);

	phone3_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), phone3, 0,1, 2,3, GTK_EXPAND, GTK_EXPAND,4,2);
	gtk_table_attach_defaults(GTK_TABLE(table), phone3_f, 1,2, 2,3);
	gtk_widget_show(alignment);
	gtk_widget_show(phone3);
	gtk_widget_show(phone3_f);

	alignment = gtk_alignment_new(0,0,0,0);
	submenu = gtk_menu_new();
	index=0;
	while ( phoneLabels[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(phoneLabels[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PhoneLabelChange4), (gpointer) index);
		index++;
	}
	phone4 = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (phone4), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (phone4), 3);

	phone4_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), phone4, 0,1, 3,4, GTK_EXPAND, GTK_EXPAND,4,2);
	gtk_table_attach_defaults(GTK_TABLE(table), phone4_f, 1,2, 3,4);
	gtk_widget_show(alignment);
	gtk_widget_show(phone4);
	gtk_widget_show(phone4_f);

	alignment = gtk_alignment_new(0,0,0,0);
	submenu = gtk_menu_new();
	index=0;
	while ( phoneLabels[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(phoneLabels[index]);
		gtk_widget_set_name (menuitem, "option_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PhoneLabelChange5), (gpointer) index);
		index++;
	}
	phone5 = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (phone5), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (phone5), 4);

	phone5_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), phone5, 0,1, 4,5, GTK_EXPAND, GTK_EXPAND,4,2);
	gtk_table_attach_defaults(GTK_TABLE(table), phone5_f, 1,2, 4,5);
	gtk_widget_show(alignment);
	gtk_widget_show(phone5);
	gtk_widget_show(phone5_f);

	gtk_widget_show(table);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, tabLabel_2);

	if (ptr)
	{
		index = FindPhoneLabel(ptr->phoneLabel1);
		gtk_option_menu_set_history (GTK_OPTION_MENU (phone1), index);
		phoneLabel[0]=index;
		index = FindPhoneLabel(ptr->phoneLabel2);
		gtk_option_menu_set_history (GTK_OPTION_MENU (phone2), index);
		phoneLabel[1]=index;
		index = FindPhoneLabel(ptr->phoneLabel3);
		gtk_option_menu_set_history (GTK_OPTION_MENU (phone3), index);
		phoneLabel[2]=index;
		index = FindPhoneLabel(ptr->phoneLabel4);
		gtk_option_menu_set_history (GTK_OPTION_MENU (phone4), index);
		phoneLabel[3]=index;
		index = FindPhoneLabel(ptr->phoneLabel5);
		gtk_option_menu_set_history (GTK_OPTION_MENU (phone5), index);
		phoneLabel[4]=index;
		gtk_entry_set_text(GTK_ENTRY(phone1_f), ptr->phone1);
		gtk_entry_set_text(GTK_ENTRY(phone2_f), ptr->phone2);
		gtk_entry_set_text(GTK_ENTRY(phone3_f), ptr->phone3);
		gtk_entry_set_text(GTK_ENTRY(phone4_f), ptr->phone4);
		gtk_entry_set_text(GTK_ENTRY(phone5_f), ptr->phone5);
	}

	/*
	 * Populate the third tab with the Custom fields.
	 */
	vbox = gtk_vbox_new(FALSE,0);
	tabLabel_3 = gtk_label_new(XP_S_ADDR_EDIT_PG_3);
	table = gtk_table_new(4,2,FALSE);
	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
	gtk_widget_show(table);

	alignment = gtk_alignment_new(0,0,0,0);
	custom1 = gtk_label_new(XP_S_CUSTOM1);
	custom1_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 0,1, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), custom1);
	gtk_table_attach_defaults(GTK_TABLE(table), custom1_f, 1,2, 0,1);
	gtk_widget_show(alignment);
	gtk_widget_show(custom1);
	gtk_widget_show(custom1_f);

	alignment = gtk_alignment_new(0,0,0,0);
	custom2 = gtk_label_new(XP_S_CUSTOM2);
	custom2_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 1,2, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), custom2);
	gtk_table_attach_defaults(GTK_TABLE(table), custom2_f, 1,2, 1,2);
	gtk_widget_show(alignment);
	gtk_widget_show(custom2);
	gtk_widget_show(custom2_f);

	alignment = gtk_alignment_new(0,0,0,0);
	custom3 = gtk_label_new(XP_S_CUSTOM3);
	custom3_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 2,3, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), custom3);
	gtk_table_attach_defaults(GTK_TABLE(table), custom3_f, 1,2, 2,3);
	gtk_widget_show(alignment);
	gtk_widget_show(custom3);
	gtk_widget_show(custom3_f);

	alignment = gtk_alignment_new(0,0,0,0);
	custom4 = gtk_label_new(XP_S_CUSTOM4);
	custom4_f = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(table), alignment, 0,1, 3,4, GTK_FILL, GTK_FILL, 4, 2);
	gtk_container_add (GTK_CONTAINER (alignment), custom4);
	gtk_table_attach_defaults(GTK_TABLE(table), custom4_f, 1,2, 3,4);
	gtk_widget_show(alignment);
	gtk_widget_show(custom4);
	gtk_widget_show(custom4_f);

	gtk_widget_show(vbox);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, tabLabel_3);

	frame = gtk_frame_new(XP_S_NOTE_TEXT);
	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	text_f = gtk_text_new(NULL, NULL);
	gtk_container_add (GTK_CONTAINER (frame), text_f);
	gtk_text_set_editable (GTK_TEXT (text_f), TRUE);
	gtk_text_set_word_wrap(GTK_TEXT(text_f), TRUE);
	gtk_widget_show(text_f);

	if (ptr)
	{
		gtk_entry_set_text(GTK_ENTRY(custom1_f), ptr->custom1);
		gtk_entry_set_text(GTK_ENTRY(custom2_f), ptr->custom2);
		gtk_entry_set_text(GTK_ENTRY(custom3_f), ptr->custom3);
		gtk_entry_set_text(GTK_ENTRY(custom4_f), ptr->custom4);

		gtk_text_freeze(GTK_TEXT(text_f));
		gtk_text_insert (
			GTK_TEXT (text_f),		/* widget */
			NULL, 						/* font */
			&text_f->style->black, 	/* fg color */
			NULL,							/* bg color */
			ptr->note, 					/* text to insert */
			-1);							/* -1: use strlen(); >0: # chars to insert */
		gtk_text_thaw (GTK_TEXT (text_f));
	}

	/* Make the dialog visible */
	gtk_widget_show(notebook);
	gtk_widget_realize(addr_editor_dialog);
	XPCenterWindow(addr_editor_dialog, NULL, addr_editor_dialog);
	gtk_widget_show(addr_editor_dialog);
}

/*========================================================================
 *	Name:			XPCloseAddrEditor()
 *	Prototype:	void XPCloseAddrEditor()
 *					
 *	Description:
 * Close the Editor window.
 *
 *	Notes:
 *========================================================================*/
void
XPCloseAddrEditor()
{

	if ( addr_editor_dialog != NULL )
	{
		gtk_widget_destroy( addr_editor_dialog );
		addr_editor_dialog = NULL;
	}
}

/*========================================================================
 *	Name:			FindPhoneLabel()
 *	Prototype:	void FindPhoneLabel()
 *					
 *	Description:
 * Match an address records phone label with the menu options.
 *
 *	Notes:
 *========================================================================*/
static int
FindPhoneLabel(
	char	*name
)
{
	int	index;
	
	index=0;
	while ( phoneLabels[index] != NULL )
	{
		if ( strcasecmp(phoneLabels[index], name) == 0 )
			return(index);
		index++;
	}

	/* This should never happen, but it saves our butt if it does. */
	return(0);
}

/*========================================================================
 *	Name:			CategoryChange()
 *	Prototype:	void CategoryChange()
 *					
 *	Description:
 * Keep track of the category for this record.
 *
 *	Notes:
 *========================================================================*/
static void
CategoryChange(
	PilotCategory_T	*ptr
)
{
	category_current = ptr;
}

/*========================================================================
 *	Name:			PhoneLabelChange1-5()
 *	Prototype:	void PhoneLabelChange1-5()
 *					
 *	Description:
 * Keep track of the phone labels selected by the user.
 *
 *	Notes:
 *========================================================================*/
static void
PhoneLabelChange1(
	int			data
)
{
	phoneLabel[0] = (int)data;
}
static void
PhoneLabelChange2(
	int			data
)
{
	phoneLabel[1] = (int)data;
}
static void
PhoneLabelChange3(
	int			data
)
{
	phoneLabel[2] = (int)data;
}
static void
PhoneLabelChange4(
	int			data
)
{
	phoneLabel[3] = (int)data;
}
static void
PhoneLabelChange5(
	int			data
)
{
	phoneLabel[4] = (int)data;
}

/*========================================================================
 *	Name:			ProcessChanges()
 *	Prototype:	void ProcessChanges()
 *					
 *	Description:
 * Retrieve the data from the dialog and either update the record or create
 * a new one.
 *
 *	Notes:
 *========================================================================*/
static void
ProcessChanges(
	GtkWidget	*dummy,
	gpointer		data
)
{
	char		*last, *first, *title, *company;
	char		*address, *city, *state, *zip, *country;
	char		*phone1, *phone2, *phone3, *phone4, *phone5;
	char		*custom1, *custom2, *custom3, *custom4;
	char		*note;

	PilotAddr_T	*ptr;


	/* Grab all the field data */
	last = gtk_editable_get_chars(GTK_EDITABLE(lastname_f), 0, -1);
	first = gtk_editable_get_chars(GTK_EDITABLE(firstname_f), 0, -1);
	title = gtk_editable_get_chars(GTK_EDITABLE(title_f), 0, -1);
	company = gtk_editable_get_chars(GTK_EDITABLE(company_f), 0, -1);
	address = gtk_editable_get_chars(GTK_EDITABLE(address_f), 0, -1);
	city = gtk_editable_get_chars(GTK_EDITABLE(city_f), 0, -1);
	state = gtk_editable_get_chars(GTK_EDITABLE(state_f), 0, -1);
	zip = gtk_editable_get_chars(GTK_EDITABLE(zipcode_f), 0, -1);
	country = gtk_editable_get_chars(GTK_EDITABLE(country_f), 0, -1);

	phone1 = gtk_editable_get_chars(GTK_EDITABLE(phone1_f), 0, -1);
	phone2 = gtk_editable_get_chars(GTK_EDITABLE(phone2_f), 0, -1);
	phone3 = gtk_editable_get_chars(GTK_EDITABLE(phone3_f), 0, -1);
	phone4 = gtk_editable_get_chars(GTK_EDITABLE(phone4_f), 0, -1);
	phone5 = gtk_editable_get_chars(GTK_EDITABLE(phone5_f), 0, -1);

	custom1 = gtk_editable_get_chars(GTK_EDITABLE(custom1_f), 0, -1);
	custom2 = gtk_editable_get_chars(GTK_EDITABLE(custom2_f), 0, -1);
	custom3 = gtk_editable_get_chars(GTK_EDITABLE(custom3_f), 0, -1);
	custom4 = gtk_editable_get_chars(GTK_EDITABLE(custom4_f), 0, -1);

	note = gtk_editable_get_chars(GTK_EDITABLE(text_f), 0, -1);

	if ( data )
	{
		/* This is an update */

		ptr = (PilotAddr_T *)data;

		/*
		 * First, free up the old storage.
		 */
		if ( ptr->category != NULL ) free(ptr->category);
		if ( ptr->last != NULL ) free(ptr->last);
		if ( ptr->first != NULL ) free(ptr->first);
		if ( ptr->title != NULL ) free(ptr->title);
		if ( ptr->company != NULL ) free(ptr->company);

		if ( ptr->address != NULL ) free(ptr->address);
		if ( ptr->city != NULL ) free(ptr->city);
		if ( ptr->state != NULL ) free(ptr->state);
		if ( ptr->zip != NULL ) free(ptr->zip);
		if ( ptr->country != NULL ) free(ptr->country);

		if ( ptr->phone1 != NULL ) free(ptr->phone1);
		if ( ptr->phone2 != NULL ) free(ptr->phone2);
		if ( ptr->phone3 != NULL ) free(ptr->phone3);
		if ( ptr->phone4 != NULL ) free(ptr->phone4);
		if ( ptr->phone5 != NULL ) free(ptr->phone5);
		if ( ptr->phoneLabel1 != NULL ) free(ptr->phoneLabel1);
		if ( ptr->phoneLabel2 != NULL ) free(ptr->phoneLabel2);
		if ( ptr->phoneLabel3 != NULL ) free(ptr->phoneLabel3);
		if ( ptr->phoneLabel4 != NULL ) free(ptr->phoneLabel4);
		if ( ptr->phoneLabel5 != NULL ) free(ptr->phoneLabel5);

		if ( ptr->custom1 != NULL ) free(ptr->custom1);
		if ( ptr->custom2 != NULL ) free(ptr->custom2);
		if ( ptr->custom3 != NULL ) free(ptr->custom3);
		if ( ptr->custom4 != NULL ) free(ptr->custom4);

		if ( ptr->note != NULL ) free(ptr->note);

	}
	else
	{
		ptr = (PilotAddr_T *)XPSafeAlloc(sizeof(PilotAddr_T));
		memset(ptr,0,sizeof(PilotAddr_T));
	}

	/* Then allocate and copy in the new values */
	ptr->category = XPSafeAlloc(strlen(category_current->name) + 1);
	ptr->last = XPSafeAlloc(strlen(last) + 1);
	ptr->first = XPSafeAlloc(strlen(first) + 1);
	ptr->title = XPSafeAlloc(strlen(title) + 1);
	ptr->company = XPSafeAlloc(strlen(company) + 1);

	ptr->address = XPSafeAlloc(strlen(address) + 1);
	ptr->city = XPSafeAlloc(strlen(city) + 1);
	ptr->state = XPSafeAlloc(strlen(state) + 1);
	ptr->zip = XPSafeAlloc(strlen(zip) + 1);
	ptr->country = XPSafeAlloc(strlen(country) + 1);
	
	ptr->phone1 = XPSafeAlloc(strlen(phone1) + 1);
	ptr->phone2 = XPSafeAlloc(strlen(phone2) + 1);
	ptr->phone3 = XPSafeAlloc(strlen(phone3) + 1);
	ptr->phone4 = XPSafeAlloc(strlen(phone4) + 1);
	ptr->phone5 = XPSafeAlloc(strlen(phone5) + 1);
	ptr->phoneLabel1 = XPSafeAlloc(strlen(phoneLabels[phoneLabel[0]]) + 1);
	ptr->phoneLabel2 = XPSafeAlloc(strlen(phoneLabels[phoneLabel[1]]) + 1);
	ptr->phoneLabel3 = XPSafeAlloc(strlen(phoneLabels[phoneLabel[2]]) + 1);
	ptr->phoneLabel4 = XPSafeAlloc(strlen(phoneLabels[phoneLabel[3]]) + 1);
	ptr->phoneLabel5 = XPSafeAlloc(strlen(phoneLabels[phoneLabel[4]]) + 1);

	ptr->custom1 = XPSafeAlloc(strlen(custom1) + 1);
	ptr->custom2 = XPSafeAlloc(strlen(custom2) + 1);
	ptr->custom3 = XPSafeAlloc(strlen(custom3) + 1);
	ptr->custom4 = XPSafeAlloc(strlen(custom4) + 1);

	ptr->note = XPSafeAlloc(strlen(note) + 1);

	strcpy(ptr->category, category_current->name);
	strcpy(ptr->last, last);
	strcpy(ptr->first, first);
	strcpy(ptr->title, title);
	strcpy(ptr->company, company);

	strcpy(ptr->address, address);
	strcpy(ptr->city, city);
	strcpy(ptr->state, state);
	strcpy(ptr->zip, zip);
	strcpy(ptr->country, country);

	strcpy(ptr->phone1, phone1);
	strcpy(ptr->phone2, phone2);
	strcpy(ptr->phone3, phone3);
	strcpy(ptr->phone4, phone4);
	strcpy(ptr->phone5, phone5);

	strcpy(ptr->phoneLabel1, phoneLabels[phoneLabel[0]]);
	strcpy(ptr->phoneLabel2, phoneLabels[phoneLabel[1]]);
	strcpy(ptr->phoneLabel3, phoneLabels[phoneLabel[2]]);
	strcpy(ptr->phoneLabel4, phoneLabels[phoneLabel[3]]);
	strcpy(ptr->phoneLabel5, phoneLabels[phoneLabel[4]]);

	strcpy(ptr->custom1, custom1);
	strcpy(ptr->custom2, custom2);
	strcpy(ptr->custom3, custom3);
	strcpy(ptr->custom4, custom4);

	strcpy(ptr->note, note);

	/* If this is a new link, add it to the link list and sort again. */
	if (data == NULL)
		XPAddAddress(ptr);


	/* Cleanup */
	g_free(last);
	g_free(first);
	g_free(title);
	g_free(company);
	g_free(address);
	g_free(city);
	g_free(state);
	g_free(zip);
	g_free(country);
	g_free(phone1);
	g_free(phone2);
	g_free(phone3);
	g_free(phone4);
	g_free(phone5);
	g_free(custom1);
	g_free(custom2);
	g_free(custom3);
	g_free(custom4);
	g_free(note);

	/* Update the Address Manager */
	XPUpdateAddrList();

	/* Close the Editor */
	XPCloseAddrEditor();
}
